@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2006 Bastian Kleineidam
+# Copyright (C) 2006-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,21 +11,19 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 Cache robots.txt contents.
 """
-from linkcheck.decorators import synchronized
-import linkcheck.robotparser2
-import linkcheck.configuration
-import linkcheck.lock
-import linkcheck.url
+from .. import robotparser2, configuration, url as urlutil
+from ..decorators import synchronized
+from ..lock import get_lock
 
 
 # lock for caching
-_lock = linkcheck.lock.get_lock("robots.txt")
+_lock = get_lock("robots.txt")
 
 
 class RobotsTxt (object):
@@ -38,22 +36,22 @@ class RobotsTxt (object):
         self.cache = {}
 
     @synchronized(_lock)
-    def allows_url (self, roboturl, url, user, password, callback=None):
+    def allows_url (self, roboturl, url, proxy, user, password, callback=None):
         """
         Ask robots.txt allowance.
         """
         if roboturl not in self.cache:
-            rp = linkcheck.robotparser2.RobotFileParser(
-                                            user=user, password=password)
+            rp = robotparser2.RobotFileParser(proxy=proxy, user=user,
+                password=password)
             rp.set_url(roboturl)
             rp.read()
-            if callback is not None:
-                parts = linkcheck.url.url_split(rp.url)
+            if hasattr(callback, '__call__'):
+                parts = urlutil.url_split(rp.url)
                 host = "%s:%d" % (parts[1], parts[2])
-                useragent = linkcheck.configuration.UserAgent
+                useragent = configuration.UserAgent
                 wait = rp.get_crawldelay(useragent)
                 callback(host, wait)
             self.cache[roboturl] = rp
         else:
             rp = self.cache[roboturl]
-        return rp.can_fetch(linkcheck.configuration.UserAgent, url)
+        return rp.can_fetch(configuration.UserAgent, url)
