@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,46 +11,41 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 Handle local file: links.
 """
 
 import re
 import os
-import time
 import urlparse
 import urllib
 import urllib2
 
-import urlbase
-import linkcheck.log
-import linkcheck.checker
-import linkcheck.fileutil
-
-# if file extension lookup was unsuccessful, look at the content
-contents = {
-    "html": re.compile(r'^(?i)<(!DOCTYPE html|html|head|title)'),
-    "opera" : re.compile(r'^Opera Hotlist'),
-    "text" : re.compile(r'(?i)^# LinkChecker URL list'),
-}
+from . import urlbase, get_index_html, absolute_url, get_url_from
+from .. import log, LOG_CHECK, fileutil, strformat, url as urlutil
+from .const import WARN_FILE_MISSING_SLASH, WARN_FILE_SYSTEM_PATH, \
+    PARSE_EXTENSIONS, PARSE_CONTENTS
 
+try:
+    import sqlite3
+    has_sqlite = True
+except ImportError:
+    has_sqlite = False
 
 def get_files (dirname):
-    """
-    Get lists of files in directory. Does only allow regular files
-    and directories, no symlinks.
-    """
-    files = []
+    """Get iterator of entries in directory. Only allows regular files
+    and directories, no symlinks."""
     for entry in os.listdir(dirname):
         fullentry = os.path.join(dirname, entry)
-        if os.path.islink(fullentry) or \
-           not (os.path.isfile(fullentry) or os.path.isdir(fullentry)):
+        if os.path.islink(fullentry):
             continue
-        files.append(entry)
-    return files
+        if os.path.isfile(fullentry):
+            yield entry
+        elif os.path.isdir(fullentry):
+            yield entry+"/"
 
 
 def prepare_urlpath_for_nt (path):
@@ -73,11 +68,21 @@ def get_nt_filename (path):
     for fname in os.listdir(head):
         if fname.lower() == tail.lower():
             return os.path.join(get_nt_filename(head), fname)
-    linkcheck.log.error(linkcheck.LOG_CHECK, "could not find %r in %r",
-                        tail, head)
+    log.error(LOG_CHECK, "could not find %r in %r", tail, head)
     return path
 
 
+def is_absolute_path (path):
+    """Check if given path is absolute. On Windows absolute paths start
+    with a drive letter. On all other systems absolute paths start with
+    a slash."""
+    if os.name == 'nt':
+       return re.search(r"^[a-zA-Z]:", path)
+    return path.startswith("/")
+
+
+firefox_extension = re.compile(r'/(?i)places.sqlite$')
+
 class FileUrl (urlbase.UrlBase):
     """
     Url link with file scheme.
@@ -98,15 +103,20 @@ class FileUrl (urlbase.UrlBase):
         base_url = self.base_url
         if not (parent_url or base_ref or base_url.startswith("file:")):
             base_url = os.path.expanduser(base_url)
-            if not base_url.startswith("/"):
+            if not is_absolute_path(base_url):
                 base_url = os.getcwd()+"/"+base_url
+                if os.path.isdir(base_url):
+                    base_url += "/"
             base_url = "file://"+base_url
-        base_url = base_url.replace("\\", "/")
-        # transform c:/windows into /c|/windows
-        base_url = re.sub("^file://(/?)([a-zA-Z]):", r"file:///\2|", base_url)
+        if os.name == "nt":
+            base_url = base_url.replace("\\", "/")
+            # transform c:/windows into /c|/windows
+            base_url = re.sub("^file://(/?)([a-zA-Z]):", r"file:///\2|", base_url)
         # norm base url again after changing
         if self.base_url != base_url:
-            base_url, is_idn = linkcheck.checker.urlbase.url_norm(base_url)
+            base_url, is_idn = urlbase.url_norm(base_url)
+            if is_idn:
+                pass # XXX warn about idn use
             self.base_url = unicode(base_url)
 
     def build_url (self):
@@ -118,7 +128,7 @@ class FileUrl (urlbase.UrlBase):
         self.urlparts[3] = self.urlparts[4] = ''
         if self.is_directory() and not self.urlparts[2].endswith('/'):
             self.add_warning(_("Added trailing slash to directory."),
-                           tag="file-missing-slash")
+                           tag=WARN_FILE_MISSING_SLASH)
             self.urlparts[2] += '/'
         self.url = urlparse.urlunsplit(self.urlparts)
 
@@ -130,7 +140,7 @@ class FileUrl (urlbase.UrlBase):
         if self.is_directory():
             self.set_result(_("directory"))
         else:
-            url = linkcheck.fileutil.pathencode(self.url)
+            url = fileutil.pathencode(self.url)
             self.url_connection = urllib2.urlopen(url)
             self.check_case_sensitivity()
 
@@ -149,48 +159,35 @@ class FileUrl (urlbase.UrlBase):
                             "system path %(realpath)r. You should always use "
                             "the system path in URLs.") % \
                             {"path": path, "realpath": realpath},
-                               tag="file-system-path")
+                               tag=WARN_FILE_SYSTEM_PATH)
 
-    def get_content (self):
-        """
-        Return file content, or in case of directories a dummy HTML file
-        with links to the files.
-        """
-        if not self.valid:
-            return ""
-        if self.has_content:
-            return self.data
-        elif self.is_directory():
-            return self.get_directory_content()
+    def read_content (self):
+        """Return file content, or in case of directories a dummy HTML file
+        with links to the files."""
+        if self.is_directory():
+            data = get_index_html(get_files(self.get_os_filename()))
+            if isinstance(data, unicode):
+                data = data.encode("iso8859-1", "ignore")
         else:
-            return super(FileUrl, self).get_content()
-
-    def get_directory_content (self):
-        """
-        Get dummy HTML data for the directory content.
-
-        @return: HTML data
-        @rtype: string
-        """
-        t = time.time()
-        files = get_files(self.get_os_filename())
-        data = linkcheck.checker.get_index_html(files)
-        self.data = data.encode("iso8859-1", "ignore")
-        self.dltime = time.time() - t
-        self.dlsize = len(self.data)
-        self.has_content = True
-        return self.data
+            data = super(FileUrl, self).read_content()
+        return data
 
     def is_html (self):
         """
-        Check if file is a parseable HTML file.
+        Check if file is a HTML file.
         """
-        if linkcheck.checker.extensions['html'].search(self.url):
+        if PARSE_EXTENSIONS['html'].search(self.url):
             return True
-        if contents['html'].search(self.get_content()):
+        if PARSE_CONTENTS['html'].search(self.get_content()):
             return True
         return False
 
+    def is_css (self):
+        """
+        Check if file is a CSS file.
+        """
+        return bool(PARSE_EXTENSIONS['css'].search(self.url))
+
     def is_file (self):
         """
         This is a file.
@@ -210,7 +207,7 @@ class FileUrl (urlbase.UrlBase):
         path = self.urlparts[2]
         if os.name == 'nt':
             path = prepare_urlpath_for_nt(path)
-        return linkcheck.fileutil.pathencode(urllib.url2pathname(path))
+        return fileutil.pathencode(urllib.url2pathname(path))
 
     def is_directory (self):
         """
@@ -232,12 +229,14 @@ class FileUrl (urlbase.UrlBase):
         if self.is_directory():
             return True
         # guess by extension
-        for ro in linkcheck.checker.extensions.itervalues():
+        for ro in PARSE_EXTENSIONS.values():
             if ro.search(self.url):
                 return True
+        if firefox_extension.search(self.url):
+            return True
         # try to read content (can fail, so catch error)
         try:
-            for ro in contents.itervalues():
+            for ro in PARSE_CONTENTS.values():
                 if ro.search(self.get_content()[:30]):
                     return True
         except IOError:
@@ -251,15 +250,41 @@ class FileUrl (urlbase.UrlBase):
         if self.is_directory():
             self.parse_html()
             return
-        for key, ro in linkcheck.checker.extensions.iteritems():
+        for key, ro in PARSE_EXTENSIONS.items():
             if ro.search(self.url):
                 getattr(self, "parse_"+key)()
                 return
-        for key, ro in contents.iteritems():
+        if has_sqlite and firefox_extension.search(self.url):
+            self.parse_firefox()
+            return
+        for key, ro in PARSE_CONTENTS.items():
             if ro.search(self.get_content()[:30]):
                 getattr(self, "parse_"+key)()
                 return
 
+    def parse_firefox (self):
+        """Parse a Firefox3 bookmark file."""
+        log.debug(LOG_CHECK, "Parsing Firefox bookmarks %s", self)
+        conn = sqlite3.connect(self.get_os_filename(), timeout=0.5)
+        try:
+            c = conn.cursor()
+            try:
+                sql = """SELECT mp.url, mb.title
+                FROM moz_places mp, moz_bookmarks mb
+                WHERE mp.hidden=0 AND mp.url NOT LIKE 'place:%' AND
+                mp.id=mb.fk"""
+                c.execute(sql)
+                for url, name in c:
+                    if not name:
+                        name = url
+                    url_data = get_url_from(url, self.recursion_level+1,
+                        self.aggregate, parent_url=self.url, name=name)
+                    self.aggregate.urlqueue.put(url_data)
+            finally:
+                c.close()
+        finally:
+            conn.close()
+
     def get_intern_pattern (self):
         """
         Get pattern for intern URL matching.
@@ -267,12 +292,11 @@ class FileUrl (urlbase.UrlBase):
         @return non-empty regex pattern or None
         @rtype String or None
         """
-        absolute = linkcheck.checker.absolute_url
-        url = absolute(self.base_url, self.base_ref, self.parent_url)
+        url = absolute_url(self.base_url, self.base_ref, self.parent_url)
         if not url:
             return None
-        parts = linkcheck.strformat.url_unicode_split(url)
-        path, params = linkcheck.url.splitparams(parts[2])
+        parts = strformat.url_unicode_split(url)
+        path = urlutil.splitparams(parts[2])[0]
         segments = path.split('/')
         if not self.is_directory():
             # cut off filename to have a directory
