@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,23 +11,20 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 Handle FTP links.
 """
 
 import ftplib
-import time
 import urllib
-import cStringIO as StringIO
+from cStringIO import StringIO
 
-import linkcheck
-import proxysupport
-import httpurl
-import internpaturl
-import linkcheck.ftpparse._ftpparse as ftpparse
+from .. import log, LOG_CHECK, LinkCheckerError
+from . import proxysupport, httpurl, internpaturl, get_index_html
+from .const import WARN_FTP_MISSING_SLASH, PARSE_EXTENSIONS
 
 DEFAULT_TIMEOUT_SECS = 300
 
@@ -96,7 +93,7 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         self.aggregate.connections.wait_for_host(host)
         try:
             self.url_connection = ftplib.FTP()
-            if linkcheck.log.is_debug(linkcheck.LOG_CHECK):
+            if log.is_debug(LOG_CHECK):
                 self.url_connection.set_debuglevel(1)
             self.url_connection.connect(host)
             if _user is None:
@@ -106,15 +103,12 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             else:
                 self.url_connection.login(_user, _password)
         except EOFError, msg:
-            msg = str(msg)
-            raise linkcheck.LinkCheckerError(
-                   _("Remote host has closed connection: %r") % msg)
+            raise LinkCheckerError(
+                  _("Remote host has closed connection: %(msg)s") % str(msg))
         if not self.url_connection.getwelcome():
-            self.close_connection()
-            raise linkcheck.LinkCheckerError(
-                   _("Got no answer from FTP server"))
+            raise LinkCheckerError(_("Got no answer from FTP server"))
         # don't set info anymore, this may change every time we log in
-        #self.add_info(linkcheck.strformat.unicode_safe(info))
+        #self.add_info(strformat.unicode_safe(info))
 
     def cwd (self):
         """
@@ -136,8 +130,7 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         if not self.filename:
             return
         files = self.get_files()
-        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
-            "FTP files %s", str(files))
+        log.debug(LOG_CHECK, "FTP files %s", str(files))
         if self.filename in files:
             # file found
             return
@@ -146,100 +139,78 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             if not self.url.endswith('/'):
                 self.add_warning(
                          _("Missing trailing directory slash in ftp url."),
-                         tag="ftp-missing-slash")
+                         tag=WARN_FTP_MISSING_SLASH)
                 self.url += '/'
             return
         raise ftplib.error_perm("550 File not found")
 
     def get_files (self):
-        """
-        Get list of filenames in directory. Subdirectories have an
-        ending slash.
-        """
+        """Get list of filenames in directory. Subdirectories have an
+        ending slash."""
         files = []
         def add_entry (line):
-            """
-            Parse list line and add the entry it points to to the file list.
-            """
-            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
-                "Directory entry %r", line)
-            try:
-                fpo = ftpparse.parse(line)
-                name = fpo.name
-                if fpo.trycwd:
+            """Parse list line and add the entry it points to to the file
+            list."""
+            log.debug(LOG_CHECK, "Directory entry %r", line)
+            from ..ftpparse import ftpparse
+            fpo = ftpparse(line)
+            if fpo is not None and fpo["name"]:
+                name = fpo["name"]
+                if fpo["trycwd"]:
                     name += "/"
-                if fpo.trycwd or fpo.tryretr:
+                if fpo["trycwd"] or fpo["tryretr"]:
                     files.append(name)
-            except (ValueError, AttributeError), msg:
-                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
-                    "%s (%s)", str(msg), line)
         self.url_connection.dir(add_entry)
         return files
 
     def is_html (self):
-        """
-        See if URL target is a HTML file by looking at the extension.
-        """
-        if linkcheck.checker.extensions['html'].search(self.url):
-            return True
-        return False
+        """See if URL target is a HTML file by looking at the extension."""
+        return bool(PARSE_EXTENSIONS['html'].search(self.url))
+
+    def is_css (self):
+        """See if URL target is a CSS file by looking at the extension."""
+        return bool(PARSE_EXTENSIONS['css'].search(self.url))
 
     def is_parseable (self):
-        """
-        See if URL target is parseable for recursion.
-        """
+        """See if URL target is parseable for recursion."""
         if self.is_directory():
             return True
-        for ro in linkcheck.checker.extensions.itervalues():
+        for ro in PARSE_EXTENSIONS.values():
             if ro.search(self.url):
                 return True
         return False
 
     def is_directory (self):
-        """
-        See if URL target is a directory.
-        """
+        """See if URL target is a directory."""
         return self.url.endswith('/')
 
     def parse_url (self):
-        """
-        Parse URL target for links.
-        """
+        """Parse URL target for links."""
         if self.is_directory():
             self.parse_html()
             return
-        for key, ro in linkcheck.checker.extensions.iteritems():
+        for key, ro in PARSE_EXTENSIONS.items():
             if ro.search(self.url):
                 getattr(self, "parse_"+key)()
 
-    def get_content (self):
-        """
-        Return URL target content, or in case of directories a dummy HTML
-        file with links to the files.
-        """
-        if not self.valid:
-            return ""
-        if self.has_content:
-            return self.data
-        t = time.time()
+    def read_content (self):
+        """Return URL target content, or in case of directories a dummy HTML
+        file with links to the files."""
         if self.is_directory():
             self.url_connection.cwd(self.filename)
             self.files = self.get_files()
-            self.data = linkcheck.checker.get_index_html(self.files)
+            data = get_index_html(self.files)
         else:
             # download file in BINARY mode
             ftpcmd = "RETR %s" % self.filename
-            buf = StringIO.StringIO()
+            buf = StringIO()
             def stor_data (s):
                 """Helper method storing given data"""
                 buf.write(s)
             self.url_connection.retrbinary(ftpcmd, stor_data)
-            self.data = buf.getvalue()
+            data = buf.getvalue()
             buf.close()
-        self.dltime = time.time() - t
-        self.dlsize = len(self.data)
-        self.has_content = True
-        return self.data
+        return data
 
     def close_connection (self):
         """
