@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,9 +11,9 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 Handle nntp: and news: links.
 """
@@ -24,9 +24,9 @@ import sys
 import nntplib
 import random
 
-import linkcheck
-import urlbase
-import linkcheck.log
+from . import urlbase
+from .. import log, LinkCheckerError, LOG_CHECK
+from .const import WARN_NNTP_NO_SERVER, WARN_NNTP_NO_NEWSGROUP, WARN_NNTP_BUSY
 
 random.seed()
 
@@ -44,27 +44,27 @@ class NntpUrl (urlbase.UrlBase):
         if not nntpserver:
             self.add_warning(
                     _("No NNTP server was specified, skipping this URL."),
-                    tag="nntp-no-server")
+                    tag=WARN_NNTP_NO_SERVER)
             return
         nntp = self._connect_nntp(nntpserver)
         group = self.urlparts[2]
         while group[:1] == '/':
             group = group[1:]
         if '@' in group:
-            # request article
-            resp, number, mid = nntp.stat("<"+group+">")
-            self.add_info(_('Articel number %s found.') % number)
+            # request article info (resp, number mid)
+            number = nntp.stat("<"+group+">")[1]
+            self.add_info(_('Article number %(num)s found.') % {"num": number})
         else:
             # split off trailing articel span
             group = group.split('/', 1)[0]
             if group:
-                # request group info
-                resp, count, first, last, name = nntp.group(group)
-                self.add_info(_("News group %s found.") % name)
+                # request group info (resp, count, first, last, name)
+                name = nntp.group(group)[4]
+                self.add_info(_("News group %(name)s found.") % {"name": name})
             else:
                 # group name is the empty string
                 self.add_warning(_("No newsgroup specified in NNTP URL."),
-                            tag="nttp-no-newsgroup")
+                            tag=WARN_NNTP_NO_NEWSGROUP)
 
     def _connect_nntp (self, nntpserver):
         """
@@ -74,7 +74,7 @@ class NntpUrl (urlbase.UrlBase):
         """
         tries = 0
         nntp = value = None
-        while tries < 5:
+        while tries < 3:
             tries += 1
             try:
                 nntp = nntplib.NNTP(nntpserver, usenetrc=False)
@@ -85,11 +85,14 @@ class NntpUrl (urlbase.UrlBase):
                 else:
                     raise
         if nntp is None:
-            raise linkcheck.LinkCheckerError(
-               _("NTTP server too busy; tried more than %d times.") % tries)
+            raise LinkCheckerError(
+               _("NNTP server too busy; tried more than %d times.") % tries)
         if value is not None:
-            self.add_warning(_("NNTP busy: %s.") % str(value),
-                             tag="nttp-busy")
+            self.add_warning(_("NNTP busy: %(msg)s.") % {"msg": str(value)},
+                             tag=WARN_NNTP_BUSY)
+        if log.is_debug(LOG_CHECK):
+            nntp.set_debuglevel(1)
+        self.add_info(nntp.getwelcome())
         return nntp
 
     def can_get_content (self):
