@@ -17,14 +17,16 @@
 """DNS rdatasets (an rdataset is a set of rdatas of a given type and class)"""
 
 import random
-import StringIO
+from cStringIO import StringIO
 import struct
 
 import linkcheck.dns.exception
 import linkcheck.dns.rdatatype
 import linkcheck.dns.rdataclass
 import linkcheck.dns.rdata
-import linkcheck.dns.set
+
+# define SimpleSet here for backwards compatibility
+SimpleSet = set
 
 class DifferingCovers(linkcheck.dns.exception.DNSException):
     """Raised if an attempt is made to add a SIG/RRSIG whose covered type
@@ -35,7 +37,7 @@ class IncompatibleTypes(linkcheck.dns.exception.DNSException):
     """Raised if an attempt is made to add rdata of an incompatible type."""
     pass
 
-class Rdataset(linkcheck.dns.set.Set):
+class Rdataset(set):
     """A DNS rdataset.
 
     @ivar rdclass: The class of the rdataset
@@ -100,12 +102,12 @@ class Rdataset(linkcheck.dns.set.Set):
         @param ttl: The TTL
         @type ttl: int"""
 
-        #
+
         # If we're adding a signature, do some special handling to
         # check that the signature covers the same type as the
         # other rdatas in this rdataset.  If this is the first rdata
         # in the set, initialize the covers field.
-        #
+
         if self.rdclass != rd.rdclass or self.rdtype != rd.rdtype:
             raise IncompatibleTypes
         if not ttl is None:
@@ -188,17 +190,17 @@ class Rdataset(linkcheck.dns.set.Set):
         else:
             ntext = ''
             pad = ''
-        s = StringIO.StringIO()
+        s = StringIO()
         if not override_rdclass is None:
             rdclass = override_rdclass
         else:
             rdclass = self.rdclass
         if len(self) == 0:
-            #
+
             # Empty rdatasets are used for the question section, and in
             # some dynamic updates, so we don't need to print out the TTL
             # (which is meaningless anyway).
-            #
+
             print >> s, '%s%s%s %s' % (ntext, pad,
                                        linkcheck.dns.rdataclass.to_text(rdclass),
                                        linkcheck.dns.rdatatype.to_text(self.rdtype))
@@ -208,9 +210,9 @@ class Rdataset(linkcheck.dns.set.Set):
                       (ntext, pad, self.ttl, linkcheck.dns.rdataclass.to_text(rdclass),
                        linkcheck.dns.rdatatype.to_text(self.rdtype),
                        rd.to_text(origin=origin, relativize=relativize, **kw))
-        #
+
         # We strip off the final \n for the caller's convenience in printing
-        #
+
         return s.getvalue()[:-1]
 
     def to_wire(self, name, file, compress=None, origin=None,
