@@ -34,7 +34,7 @@ import sys
 import socket
 import errno
 import cgi
-import cStringIO as StringIO
+from cStringIO import StringIO
 
 # Set various FastCGI constants
 # Maximum number of requests that can be handled
@@ -88,32 +88,24 @@ error = 'fcgi.error'
 # anywhere at the moment
 
 def _error (msg):
-    """
-    Append a string to /tmp/err.
-    """
+    """Append a string to /tmp/err."""
     errf = file('/tmp/err', 'a+')
     errf.write(msg+'\n')
     errf.close()
 
 
 class Record (object):
-    """
-    Class representing FastCGI records.
-    """
+    """Class representing FastCGI records."""
 
     def __init__ (self):
-        """
-        Initialize record data.
-        """
+        """Initialize record data."""
         self.version = FCGI_VERSION_1
         self.rec_type = FCGI_UNKNOWN_TYPE
         self.req_id = FCGI_NULL_REQUEST_ID
         self.content = ""
 
     def read_record (self, sock):
-        """
-        Read a FastCGI record from socket.
-        """
+        """Read a FastCGI record from socket."""
         s = [ord(x) for x in sock.recv(8)]
         self.version, self.rec_type, padding_length = s[0], s[1], s[6]
         self.req_id, content_length = (s[2]<<8)+s[3], (s[4]<<8)+s[5]
@@ -145,9 +137,7 @@ class Record (object):
             self.protocolStatus = ord(c[4])
 
     def write_record (self, sock):
-        """
-        Write a FastCGI request to socket.
-        """
+        """Write a FastCGI request to socket."""
         content = self.content
         if self.rec_type == FCGI_BEGIN_REQUEST:
             content = chr(self.role>>8) + chr(self.role & 255) + \
@@ -178,7 +168,7 @@ class Record (object):
                 c_len & 255,
                 pad_len,
                 0]
-        hdr = ''.join([chr(x) for x in hdr])
+        hdr = ''.join(chr(x) for x in hdr)
 
         sock.send(hdr + content + pad_len*'\000')
 
@@ -224,86 +214,66 @@ def HandleManTypes (r, conn):
                 'FCGI_MAX_REQS'  : FCGI_MAX_REQS,
                 'FCGI_MPXS_CONNS': FCGI_MPXS_CONNS}
         for i in r.values.keys():
-            if _vars.has_key(i):
+            if i in _vars:
                 v[i] = _vars[i]
         r.values = _vars
         r.write_record(conn)
 
 
 class FastCGIWriter (object):
-    """
-    File-like object writing FastCGI requests. All read operations
-    return empty data.
-    """
+    """File-like object writing FastCGI requests. All read operations
+    return empty data."""
 
     def __init__ (self, rec, conn):
-        """
-        Initialize with given record and connection.
-        """
+        """Initialize with given record and connection."""
         self.record = rec
         self.conn = conn
         self.closed = False
 
     def close (self):
-        """
-        Close this writer.
-        """
+        """Close this writer."""
         if not self.closed:
             self.closed = True
             self.record.content = ""
             self.record.write_record(self.conn)
 
     def isatty (self):
-        """
-        Returns False.
-        """
+        """Returns False."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
         return False
 
     def seek (self, pos, mode=0):
-        """
-        Does nothing.
-        """
+        """Does nothing."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
 
     def tell (self):
-        """
-        Return zero.
-        """
+        """Return zero."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
         return 0
 
     def read (self, n=-1):
-        """
-        Return empty string.
-        """
+        """Return empty string."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
         return ""
 
     def readline (self, length=None):
-        """
-        Return empty string.
-        """
+        """Return empty string."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
         return ""
 
     def readlines (self):
-        """
-        Return empty list.
-        """
+        """Return empty list."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
         return []
 
     def write (self, s):
-        """
-        Write data in record for record to connection.
-        """
+        """Write data in record for record to connection."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
         while s:
@@ -312,23 +282,17 @@ class FastCGIWriter (object):
             self.record.write_record(self.conn)
 
     def get_next_chunk (self, data):
-        """
-        Return tuple (chunk of data, newdata).
-        """
+        """Return tuple (chunk of data, newdata)."""
         chunk = data[:8192]
         data = data[8192:]
         return chunk, data
 
     def writelines (self, lines):
-        """
-        Write given lines to the connection.
-        """
+        """Write given lines to the connection."""
         self.write(''.join(lines))
 
     def flush (self):
-        """
-        Does nothing.
-        """
+        """Raises ValueError if called with closed file."""
         if self.closed:
             raise ValueError, "I/O operation on closed file"
 
@@ -397,7 +361,7 @@ class FCGI (object):
             self.env = os.environ
             self.stdin, self.out, self.err = sys.stdin, sys.stdout, sys.stderr
 
-        if os.environ.has_key('FCGI_WEB_SERVER_ADDRS'):
+        if 'FCGI_WEB_SERVER_ADDRS' in os.environ:
             addrs = os.environ['FCGI_WEB_SERVER_ADDRS'].split(',')
             good_addrs = [ addr.strip() for addr in addrs ]
         else:
@@ -405,7 +369,7 @@ class FCGI (object):
 
         self.conn, addr = _sock.accept()
         # Check if the connection is from a legal address
-        if good_addrs != None and addr not in good_addrs:
+        if good_addrs is not None and addr not in good_addrs:
             raise error, 'Connection from invalid server!'
 
         stdin = data = ""
@@ -465,8 +429,8 @@ class FCGI (object):
                     data += r.content
         # end of while remaining:
 
-        self.stdin = sys.stdin  = StringIO.StringIO(stdin)
-        self.data = StringIO.StringIO(data)
+        self.stdin = sys.stdin  = StringIO(stdin)
+        self.data = StringIO(data)
         r = Record()
         r.rec_type = FCGI_STDERR
         r.req_id = self.request_id
@@ -494,7 +458,7 @@ class FCGI (object):
 
     def getFieldStorage (self):
         method = 'GET'
-        if self.env.has_key('REQUEST_METHOD'):
+        if 'REQUEST_METHOD' in self.env:
             method = self.env['REQUEST_METHOD'].upper()
         if method == 'GET':
             return cgi.FieldStorage(environ=self.env, keep_blank_values=1)
