@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,20 +11,17 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 A HTML logger.
 """
-
 import time
 import cgi
 import os
-
-import linkcheck.logger
-import linkcheck.strformat
-import linkcheck.configuration
+from . import Logger
+from .. import strformat, configuration
 
 
 # ss=1 enables show source
@@ -37,8 +34,8 @@ HTML_HEADER = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
  "http://www.w3.org/TR/html4/loose.dtd">
 <html>
 <head>
-<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1">
-<title>%s</title>
+<meta http-equiv="Content-Type" content="text/html; charset=%(encoding)s">
+<title>%(title)s</title>
 <style type="text/css">
 <!--
  h2 { font-family: Verdana,sans-serif; font-size: 22pt; font-weight: bold }
@@ -49,10 +46,10 @@ HTML_HEADER = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
 -->
 </style>
 </head>
-<body bgcolor="%s" link="%s" vlink="%s" alink="%s">
+<body bgcolor="%(body)s" link="%(link)s" vlink="%(vlink)s" alink="%(alink)s">
 """
 
-class HtmlLogger (linkcheck.logger.Logger):
+class HtmlLogger (Logger):
     """
     Logger with HTML output.
     """
@@ -86,21 +83,25 @@ class HtmlLogger (linkcheck.logger.Logger):
         self.write(u" -->")
 
     def start_output (self):
-        """
-        Write start of checking info.
-        """
+        """Write start of checking info."""
         super(HtmlLogger, self).start_output()
         self.starttime = time.time()
-        self.write(HTML_HEADER % (linkcheck.configuration.App,
-                   self.colorbackground,
-                   self.colorlink, self.colorlink, self.colorlink))
-        self.comment("Generated by %s" % linkcheck.configuration.App)
+        header = {
+            "encoding": self.output_encoding,
+            "title": configuration.App,
+            "body": self.colorbackground,
+            "link": self.colorlink,
+            "vlink": self.colorlink,
+            "alink": self.colorlink,
+        }
+        self.write(HTML_HEADER % header)
+        self.comment("Generated by %s" % configuration.App)
         if self.has_part('intro'):
-            self.write(u"<center><h2>"+linkcheck.configuration.App+
+            self.write(u"<center><h2>"+configuration.App+
                        "</h2></center><br><blockquote>"+
-                       linkcheck.configuration.Freeware+"<br><br>"+
+                       configuration.Freeware+"<br><br>"+
                        (_("Start checking at %s") %
-                       linkcheck.strformat.strtime(self.starttime))+
+                       strformat.strtime(self.starttime))+
                        os.linesep+"<br>")
             self.check_date()
         self.flush()
@@ -110,6 +111,8 @@ class HtmlLogger (linkcheck.logger.Logger):
         Write url checking info as HTML.
         """
         self.write_table_start()
+        if self.has_part("id"):
+            self.write_id()
         if self.has_part("url"):
             self.write_url(url_data)
         if url_data.name and self.has_part("name"):
@@ -151,20 +154,22 @@ class HtmlLogger (linkcheck.logger.Logger):
                      (cgi.escape(_("checked link")), self.colorbackground))
 
     def write_table_end (self):
-        """
-        End html table.
-        """
+        """End html table."""
         self.write(u'</table></td></tr></table><br clear="all">')
 
+    def write_id (self):
+        """Write ID for current URL."""
+        self.writeln(u"<tr>")
+        self.writeln(u'<td>%s</td>' % self.part("id"))
+        self.write(u"<td>%d</td></tr>" % self.number)
+
     def write_url (self, url_data):
-        """
-        Write url_data.base_url.
-        """
+        """Write url_data.base_url."""
         self.writeln(u"<tr>")
         self.writeln(u'<td bgcolor="%s">%s</td>' %
                      (self.colorurl, self.part("url")))
         self.write(u'<td bgcolor="%s">' % self.colorurl)
-        self.write(u"%s" % cgi.escape(repr(url_data.base_url or u"")[1:]))
+        self.write(u"`%s'" % cgi.escape(url_data.base_url))
         if url_data.cached:
             self.write(_(" (cached)"))
         self.writeln(u"</td></tr>")
@@ -173,8 +178,8 @@ class HtmlLogger (linkcheck.logger.Logger):
         """
         Write url_data.name.
         """
-        self.writeln(u"<tr><td>" + self.part("name") + u"</td><td>" +
-                     cgi.escape(repr(url_data.name)[1:]) + u"</td></tr>")
+        args = (self.part("name"), cgi.escape(url_data.name))
+        self.writeln(u"<tr><td>%s</td><td>`%s'</td></tr>" % args)
 
     def write_parent (self, url_data):
         """
@@ -223,7 +228,7 @@ class HtmlLogger (linkcheck.logger.Logger):
         Write url_data.dlsize.
         """
         self.writeln(u"<tr><td>"+self.part("dlsize")+u"</td><td>"+
-                     linkcheck.strformat.strsize(url_data.dlsize)+
+                     strformat.strsize(url_data.dlsize)+
                      u"</td></tr>")
 
     def write_checktime (self, url_data):
@@ -238,17 +243,16 @@ class HtmlLogger (linkcheck.logger.Logger):
         Write url_data.info.
         """
         sep = u"<br>"+os.linesep
-        text = sep.join([cgi.escape(x[1]) for x in url_data.info])
+        text = sep.join(cgi.escape(x) for x in url_data.info)
         self.writeln(u'<tr><td valign="top">' + self.part("info")+
-               u"</td><td>"+cgi.escape(text).replace(os.linesep, "<br>")+
-               u"</td></tr>")
+               u"</td><td>"+text+u"</td></tr>")
 
     def write_warning (self, url_data):
         """
         Write url_data.warnings.
         """
         sep = u"<br>"+os.linesep
-        text = sep.join([cgi.escape(x[1]) for x in url_data.warnings])
+        text = sep.join(cgi.escape(x) for x in url_data.warnings)
         self.writeln(u'<tr><td bgcolor="' + self.colorwarning + u'" '+
                      u'valign="top">' + self.part("warning") +
                      u'</td><td bgcolor="' + self.colorwarning + u'">' +
@@ -283,24 +287,31 @@ class HtmlLogger (linkcheck.logger.Logger):
                 self.write(_n("%d link checked.", "%d links checked.",
                            self.number) % self.number)
                 self.write(u" ")
-            self.write(_n("%d warning found.", "%d warnings found.",
-                            self.warnings) % self.warnings)
-            self.write(u" ")
-            self.writeln(_n("%d error found.", "%d errors found.",
-                         self.errors) % self.errors)
+            self.write(_n("%d warning found", "%d warnings found",
+                            self.warnings_printed) % self.warnings_printed)
+            if self.warnings != self.warnings_printed:
+                self.write(_(" (%d ignored or duplicates not printed)") %
+                    (self.warnings - self.warnings_printed))
+            self.write(u". ")
+            self.write(_n("%d error found", "%d errors found",
+                         self.errors_printed) % self.errors_printed)
+            if self.errors != self.errors_printed:
+                self.write(_(" (%d duplicates not printed)") %
+                    (self.errors - self.errors_printed))
+            self.writeln(u".")
             self.writeln(u"<br>")
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
             self.writeln(_("Stopped checking at %(time)s (%(duration)s)") %
-                 {"time": linkcheck.strformat.strtime(self.stoptime),
-                  "duration": linkcheck.strformat.strduration_long(duration)})
+                 {"time": strformat.strtime(self.stoptime),
+                  "duration": strformat.strduration_long(duration)})
             self.writeln(u'</blockquote><br><hr noshade size="1"><small>'+
-                         linkcheck.configuration.HtmlAppInfo+u"<br>")
+                         configuration.HtmlAppInfo+u"<br>")
             self.writeln(_("Get the newest version at %s") %
-             (u'<a href="'+linkcheck.configuration.Url+u'" target="_top">'+
-              linkcheck.configuration.Url+u"</a>.<br>"))
+               (u'<a href="'+configuration.Url+u'" target="_top">'+
+                configuration.Url+u"</a>.<br>"))
             self.writeln(_("Write comments and bugs to %s") %
-             (u'<a href="mailto:'+linkcheck.configuration.Email+u'">'+
-             linkcheck.configuration.Email+u"</a>."))
+               (u'<a href="mailto:'+configuration.Email+u'">'+
+                configuration.Email+u"</a>."))
             self.writeln(u"</small></body></html>")
         self.close_fileoutput()
