@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,23 +11,18 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 The default text logger.
 """
-
 import time
-
-import linkcheck
-import linkcheck.ansicolor
-import linkcheck.logger
-import linkcheck.strformat
-import linkcheck.configuration
+from . import Logger
+from .. import ansicolor, strformat, configuration, i18n
 
 
-class TextLogger (linkcheck.logger.Logger):
+class TextLogger (Logger):
     """
     A text logger, colorizing the output if possible.
 
@@ -42,7 +37,7 @@ class TextLogger (linkcheck.logger.Logger):
     as a default and then switch to another configured output. So we
     must not print anything out at __init__ time.
 
-    C{def log_filter_url (self, url_data, do_filter)}
+    C{def log_filter_url (self, url_data, do_print)}
     Called every time an url finished checking. All data we checked is in
     the UrlData object url_data.
 
@@ -69,25 +64,30 @@ class TextLogger (linkcheck.logger.Logger):
         Initialize error counter and optional file output.
         """
         super(TextLogger, self).__init__(**args)
+        self.output_encoding = args.get("encoding", i18n.default_encoding)
         self.init_fileoutput(args)
+        self.colorparent = args.get('colorparent', 'default')
+        self.colorurl = args.get('colorurl', 'default')
+        self.colorname = args.get('colorname', 'default')
+        self.colorreal = args.get('colorreal', 'default')
+        self.colorbase = args.get('colorbase', 'default')
+        self.colorvalid = args.get('colorvalid', 'default')
+        self.colorinvalid = args.get('colorinvalid', 'default')
+        self.colorinfo = args.get('colorinfo', 'default')
+        self.colorwarning = args.get('colorwarning', 'default')
+        self.colordltime = args.get('colordltime', 'default')
+        self.colordlsize = args.get('colordlsize', 'default')
+        self.colorreset = args.get('colorreset', 'default')
+
+    def init_fileoutput (self, args):
+        super(TextLogger, self).init_fileoutput(args)
         if self.fd is not None:
-            self.fd = linkcheck.ansicolor.Colorizer(self.fd)
-        self.colorparent = args['colorparent']
-        self.colorurl = args['colorurl']
-        self.colorname = args['colorname']
-        self.colorreal = args['colorreal']
-        self.colorbase = args['colorbase']
-        self.colorvalid = args['colorvalid']
-        self.colorinvalid = args['colorinvalid']
-        self.colorinfo = args['colorinfo']
-        self.colorwarning = args['colorwarning']
-        self.colordltime = args['colordltime']
-        self.colordlsize = args['colordlsize']
-        self.colorreset = args['colorreset']
+            self.fd = ansicolor.Colorizer(self.fd)
 
     def start_fileoutput (self):
+        """Needed to make file descriptor color aware."""
         super(TextLogger, self).start_fileoutput()
-        self.fd = linkcheck.ansicolor.Colorizer(self.fd)
+        self.fd = ansicolor.Colorizer(self.fd)
 
     def start_output (self):
         """
@@ -96,22 +96,24 @@ class TextLogger (linkcheck.logger.Logger):
         super(TextLogger, self).start_output()
         self.starttime = time.time()
         if self.has_part('intro'):
-            self.writeln(linkcheck.configuration.AppInfo)
-            self.writeln(linkcheck.configuration.Freeware)
-            self.writeln(_("Get the newest version at %s") %
-                         linkcheck.configuration.Url)
-            self.writeln(_("Write comments and bugs to %s") %
-                         linkcheck.configuration.Email)
+            self.writeln(configuration.AppInfo)
+            self.writeln(configuration.Freeware)
+            self.writeln(_("Get the newest version at %(url)s") %
+                         {'url': configuration.Url})
+            self.writeln(_("Write comments and bugs to %(email)s") %
+                         {'email': configuration.Email})
             self.check_date()
             self.writeln()
             self.writeln(_("Start checking at %s") %
-                         linkcheck.strformat.strtime(self.starttime))
+                         strformat.strtime(self.starttime))
             self.flush()
 
     def log_url (self, url_data):
         """
         Write url checking info.
         """
+        if self.has_part('id'):
+            self.write_id()
         if self.has_part('url'):
             self.write_url(url_data)
         if url_data.name and self.has_part('name'):
@@ -122,12 +124,12 @@ class TextLogger (linkcheck.logger.Logger):
             self.write_base(url_data)
         if url_data.url and self.has_part('realurl'):
             self.write_real(url_data)
+        if url_data.checktime and self.has_part('checktime'):
+            self.write_checktime(url_data)
         if url_data.dltime >= 0 and self.has_part('dltime'):
             self.write_dltime(url_data)
         if url_data.dlsize >= 0 and self.has_part('dlsize'):
             self.write_dlsize(url_data)
-        if url_data.checktime and self.has_part('checktime'):
-            self.write_checktime(url_data)
         if url_data.info and self.has_part('info'):
             self.write_info(url_data)
         if url_data.warnings and self.has_part('warning'):
@@ -136,13 +138,16 @@ class TextLogger (linkcheck.logger.Logger):
             self.write_result(url_data)
         self.flush()
 
-    def write_url (self, url_data):
-        """
-        Write url_data.base_url.
-        """
+    def write_id (self):
+        """Write unique ID of url_data."""
         self.writeln()
+        self.write(self.part('id') + self.spaces('id'))
+        self.writeln(u"%d" % self.number, color=self.colorinfo)
+
+    def write_url (self, url_data):
+        """Write url_data.base_url."""
         self.write(self.part('url') + self.spaces('url'))
-        txt = unicode(repr(url_data.base_url or u"")[1:])
+        txt = strformat.strline(url_data.base_url)
         if url_data.cached:
             txt += _(" (cached)")
         self.writeln(txt, color=self.colorurl)
@@ -152,7 +157,7 @@ class TextLogger (linkcheck.logger.Logger):
         Write url_data.name.
         """
         self.write(self.part("name") + self.spaces("name"))
-        self.writeln(unicode(repr(url_data.name)[1:]), color=self.colorname)
+        self.writeln(strformat.strline(url_data.name), color=self.colorname)
 
     def write_parent (self, url_data):
         """
@@ -191,7 +196,7 @@ class TextLogger (linkcheck.logger.Logger):
         Write url_data.dlsize.
         """
         self.write(self.part("dlsize") + self.spaces("dlsize"))
-        self.writeln(linkcheck.strformat.strsize(url_data.dlsize),
+        self.writeln(strformat.strsize(url_data.dlsize),
                      color=self.colordlsize)
 
     def write_checktime (self, url_data):
@@ -207,16 +212,14 @@ class TextLogger (linkcheck.logger.Logger):
         Write url_data.info.
         """
         self.write(self.part("info") + self.spaces("info"))
-        text = [x[1] for x in url_data.info]
-        self.writeln(self.wrap(text, 65), color=self.colorinfo)
+        self.writeln(self.wrap(url_data.info, 65), color=self.colorinfo)
 
     def write_warning (self, url_data):
         """
         Write url_data.warning.
         """
         self.write(self.part("warning") + self.spaces("warning"))
-        text = [x[1] for x in url_data.warnings]
-        self.writeln(self.wrap(text, 65), color=self.colorwarning)
+        self.writeln(self.wrap(url_data.warnings, 65), color=self.colorwarning)
 
     def write_result (self, url_data):
         """
@@ -244,15 +247,21 @@ class TextLogger (linkcheck.logger.Logger):
                 self.write(_n("%d link checked.", "%d links checked.",
                               self.number) % self.number)
                 self.write(u" ")
-
-            self.write(_n("%d warning found.", "%d warnings found.",
-                            self.warnings) % self.warnings)
-            self.write(u" ")
-            self.writeln(_n("%d error found.", "%d errors found.",
-                            self.errors) % self.errors)
+            self.write(_n("%d warning found", "%d warnings found",
+                            self.warnings_printed) % self.warnings_printed)
+            if self.warnings != self.warnings_printed:
+                self.write(_(" (%d ignored or duplicates not printed)") %
+                    (self.warnings - self.warnings_printed))
+            self.write(u". ")
+            self.write(_n("%d error found", "%d errors found",
+                            self.errors_printed) % self.errors_printed)
+            if self.errors != self.errors_printed:
+                self.write(_(" (%d duplicates not printed)") %
+                    (self.errors - self.errors_printed))
+            self.writeln(u".")
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
-            self.writeln(_("Stopped checking at %s (%s)") %
-                         (linkcheck.strformat.strtime(self.stoptime),
-                          linkcheck.strformat.strduration_long(duration)))
+            self.writeln(_("Stopped checking at %(time)s (%(duration)s)") %
+                 {"time": strformat.strtime(self.stoptime),
+                  "duration": strformat.strduration_long(duration)})
         self.close_fileoutput()
